
package mobicomp.hearts.lobby;


import java.util.Random;

import mobicomp.hearts.communication.HeartsCommunication;
import mobicomp.hearts.data.Player;


/**
 * This class is the Controller in the model-view-controller 
 * pattern consisting of LobbyData, LobbyFrame, and LobbyController
 * 
 * @author Andreas Wetzel <andi@spamato.net>
 */
public class LobbyController
{
    LobbyData data;	// storage object for information
    HeartsCommunication comm; // the communication facility
    LobbyFrame frame; // the gui

    /**
     * @param player The local player
     */
    public LobbyController(Player player) {
        frame = new LobbyFrame(this);
        data = new LobbyData(player);
        data.addObserver(frame);
        data.notifyObservers();
        comm = new HeartsCommunication(data);
        frame.setVisible(true);

        System.out.println("I am " + player.getId());
    }

    /**
     * Selects a specific known game
     * @param i Index of the game in the Vector of known games
     */
    public void selectOpenGame(int i){
        data.currentGame = (Game) data.openGames.get(i);
        data.notifyObservers();
    }

    /**
     * This method is used to start and abort a flooding
     * looking for open games. 
     */
    public void toggleLookForGames() {
        if ( !data.isLookingForGames ) {
            // start thread to look for open games
            comm.startLookingForOpenGames();
        }
        else {
            // stop looking thread
            // thread stops automatically if data.isLookingForGames is false
        }
        data.isLookingForGames = !data.isLookingForGames;
        data.notifyObservers();
    }

    /**
     * This method is used to start/stop hosting a game
     */
    public void toggleHostGame() {
        if ( !data.isHostingGame ) {
       
            // stop looking for games
            data.isLookingForGames = false;
            data.currentGame = new Game(data.player);
            data.openGames.clear();
            data.updateOpenGame( data.currentGame);
        }
        else {
            // stop hosting game
        	comm.abortLocaldGame();
        	
            data.openGames.clear();
            data.currentGame = null;
        }

        data.isHostingGame = !data.isHostingGame;
        data.notifyObservers();
    }

    /**
     * called if the user clicks the start/join/leave button
     *
     */
    public void changeGameState() {
        if (!data.isHostingGame) {
            // player wants to join/leave a remote game

            if (!data.hasJoined) {
                comm.joinRemoteGame( data.currentGame.getHostPlayer());
            }
            else {
                comm.leaveRemoteGame( data.currentGame.getHostPlayer());
                // toggle flag
                data.hasJoined = false;
                data.notifyObservers();
            }
           
        }
        else {
            data.currentGame.setStarted();
            data.notifyObservers();
            
            // tell clients to start game...
            comm.startGame( data.currentGame);
        }
    }

	public static void main( String[] args) {
        int id = new Random().nextInt(65000);
        Player player = new Player( id, Integer.toString( id));
        new LobbyController( player);
    }
}